/* k-means - returns id and centroids given vectors in a row-wise ordering 
 * and initial centers
 * [id,centers] = kmeans_c(X,c,maxIters)
 *
 * Copyright Ananth Ranganathan (Honda Research Institute) 2010
 */

#include "mex.h"
#include <string.h>
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    double *X, *c, *tmp;
    double *id, *centers, *cprev, c_changed=1;
    int K, maxIters, nc, iters=0;
    int m,n,i,j,k,index;
    double mindist,curdist;
    double *nums;
    
    if(nrhs!=3)
    {
        mexErrMsgTxt("Number of input arguments must be 3");
    }
    if(nlhs!=2)
    {
        mexErrMsgTxt("Number of output arguments must be 2");
    }
    
    m = mxGetM(prhs[0]);
    n = mxGetN(prhs[0]);
    K = mxGetM(prhs[1]);
    nc = mxGetN(prhs[1]);
    X = mxGetPr(prhs[0]);
    c = mxGetPr(prhs[1]);
    tmp = mxGetPr(prhs[2]);
    maxIters = *tmp;
    
    if (nc != n)
    {
        mexErrMsgTxt("Length of vector X is not the same as length of vectors in c");
    }
    
    plhs[0] = mxCreateDoubleMatrix(1,m,mxREAL);
    id = mxGetPr(plhs[0]);
    plhs[1] = mxCreateDoubleMatrix(K,n,mxREAL);
    centers = mxGetPr(plhs[1]);
    memcpy(centers,c,sizeof(double)*K*n); 
    cprev = (double*)mxMalloc(sizeof(double)*K*n);
    memset(cprev,0,sizeof(double)*K*n); 
    nums = (double*)mxMalloc(sizeof(double)*K);
    memset(nums,0,sizeof(double)*K); 
    
    
    /*loop while centers have not changed and iters<maxIters*/
    while(c_changed>1e-4 && iters<maxIters)
    {
        memcpy(cprev,centers,sizeof(double)*K*n);
        iters = iters+1;
        memset(centers,0,sizeof(double)*K*n); 
        memset(nums,0,sizeof(double)*K);
        
        /*loop over all data points*/
        for(i=0;i<m;i++)
        {
            /*loop over centroids to find closest one*/
            for(j=0;j<K;j++)
            {
                curdist = 0;
                
                for(k=0;k<n;k++)
                {
                    curdist += (X[k*m+i] - cprev[k*K+j])*(X[k*m+i] - cprev[k*K+j]);
                }
                
                if(j==0 || curdist<mindist)
                {
                    mindist = curdist;
                    id[i] = (double)j;
                }
            }
            
            /*add X(i,:) to the appropriate center and increment nums*/
            index = (int)id[i];
            nums[index]++;
            for(k=0;k<n;k++)
            {
                centers[k*K+index] += X[k*m+i];
            }
        }
        
        /*calculate centers as average*/
        for(i=0;i<K;i++)
        {
            for(j=0;j<n;j++)
            {
                if(nums[i]!=0)
                    centers[j*K+i] /= nums[i];
                else /*initialize to first data point*/
                    centers[j*K+i] = X[j*m];
            }
        }
        
        /*calculate c_changed*/
        c_changed = 0;
        for(i=0;i<K*n;i++)
        {
            c_changed += (cprev[i]-centers[i])*(cprev[i]-centers[i]);
        }
        
        mexPrintf("Iter %d Centers moved by squared distance %f\n",iters,c_changed);
        mexEvalString("pause(.0001);");
    }
    
    /*change id to base 1 like matlab*/
    for(i=0;i<m;i++)
        id[i]++;
}
