function H_all = BuildHistogramsAtLevel( level, imageFileList, dataBaseDir, featureSuffix, ...
    dictionarySize, canSkip)
%find texton labels of patches and compute histograms of all images at
%specified spatial level

%Just like BuildHistograms but only for one level

fprintf('Building Histograms at level %d\n\n', level);


%% load texton dictionary (all texton centers)
inFName = fullfile(dataBaseDir, sprintf('dictionary_%d.mat', dictionarySize));
load(inFName,'dictionary');

fprintf('Loaded texton dictionary: %d textons\n', dictionarySize);

outFName = fullfile(dataBaseDir, sprintf('HistogramsAtLevel_%d_%d',level,dictionarySize));
if(size(dir(outFName),1)~=0 && canSkip),
    t = load(outFName);
    H_all = t;
    return;
end

%% compute texton labels of patches and histograms
H_all = [];

for f = 1:size(imageFileList,1)

    imageFName = imageFileList{f};
    [dirN base] = fileparts(imageFName);
    baseFName = fullfile(dirN, base);
    inFName = fullfile(dataBaseDir, sprintf('%s%s', baseFName, featureSuffix));
    
    %load or compute histograms for the image
    outTName = fullfile(dataBaseDir, sprintf('%s_texton_ind_%d.mat', baseFName, dictionarySize));
    outHName = fullfile(dataBaseDir, sprintf('%s_HistAtLevel_%d.mat', baseFName, dictionarySize));
    if(size(dir(outTName),1)~=0 && size(dir(outHName),1)~=0 && canSkip),
        load(outHName, 'hists');
        H_all = [H_all; hists];
        continue;
    end

    %% load sift descriptors
    load(inFName, 'features');
    ndata = size(features.data,1);
    fprintf('Loaded %s, %d descriptors\n', inFName, ndata);

    %% find texton indices and compute histogram
    texton_ind.data = zeros(ndata,1);
    texton_ind.x = features.x;
    texton_ind.y = features.y;
    texton_ind.wid = features.wid;
    texton_ind.hgt = features.hgt;
    %run in batches to keep the memory foot print small
    batchSize = 10000;
    opts.scale_A = 1;
    opts.record = -1;

    if ndata <= batchSize
        dist_mat = sp_dist2(features.data, dictionary);
        [min_dist, min_ind] = min(dist_mat, [], 2);
        texton_ind.data = min_ind;
    else
        for j = 1:batchSize:ndata
            lo = j;
            hi = min(j+batchSize-1,ndata);
            dist_mat = dist2(features.data(lo:hi,:), dictionary);
            [min_dist, min_ind] = min(dist_mat, [], 2);
            texton_ind.data(lo:hi,:) = min_ind;
        end
    end
    save(outTName, 'texton_ind');
    
    %% compute histograms at that level
    binsHigh = 2^(level-1);
    wid = texton_ind.wid;
    hgt = texton_ind.hgt;
    hists = [];
    
    for i=1:binsHigh
        for j=1:binsHigh
            % find the coordinates of the current bin
            x_lo = floor(wid/binsHigh * (i-1));
            x_hi = floor(wid/binsHigh * i);
            y_lo = floor(hgt/binsHigh * (j-1));
            y_hi = floor(hgt/binsHigh * j);
            
            texton_patch = texton_ind.data( (texton_ind.x > x_lo) & (texton_ind.x <= x_hi) & ...
                                            (texton_ind.y > y_lo) & (texton_ind.y <= y_hi));
            
            % make histogram of features in bin
            hists = [hists ; hist(texton_patch, 1:dictionarySize)./length(texton_ind.data)];
        end
    end
    save(outHName,'hists');
    H_all = [H_all; hists];
end

%% save histograms of all images in this directory in a single file
save(outFName, 'H_all', '-ascii');


end