function [] = DisplayPyramid(imageFName, dataBaseDir, textonSuffix, dictionarySize, pyramidLevels)
%
% Generate the pyramid from the texton labels as in CompilePyramid but only
% for one image and for the purpose of displaying this pyramid nicely in
% a grid format
%
% imageFName: cell of file paths
% dataBaseDir: the base directory for the data files that are generated
%  by the algorithm. If this dir is the same as imageBaseDir the files
%  will be generated in the same location as the image file
% textonSuffix: this is the suffix appended to the image file name to
%  denote the data file that contains the textons indices and coordinates. 
%  Its default value is '_texton_ind_%d.mat' where %d is the dictionary
%  size.
% dictionarySize: size of descriptor dictionary (200 has been found to be
%  a good size)
% pyramidLevels: number of levels of the pyramid to build

fprintf('Building Spatial Pyramid for Display\n\n');

%% parameters

if(nargin<4)
    dictionarySize = 200
end

if(nargin<5)
    pyramidLevels = 4
end

if(nargin<6)
    canSkip = 0
end

binsHigh = 2^(pyramidLevels-1);

%% load image
[dirN base] = fileparts(imageFName);
[dirN base2] = fileparts(dirN);
image = imread(imageFName);
    
%% load texton indices
in_fname = fullfile(dataBaseDir, base2, sprintf('%s%s', base, textonSuffix));
load(in_fname, 'texton_ind');
    
%% get width and height of input image
wid = texton_ind.wid;
hgt = texton_ind.hgt;

fprintf('Loaded %s: wid %d, hgt %d\n',imageFName, wid, hgt);

%% compute histogram at the finest level
pyramid_cell = cell(pyramidLevels,1);
pyramid_cell{1} = zeros(binsHigh, binsHigh, dictionarySize);

xcoords = [];
ycoords = [];

for i=1:binsHigh
    % find the coordinates of the current bin
    x_lo = floor(wid/binsHigh * (i-1));
    x_hi = floor(wid/binsHigh * i);
    xcoords = [xcoords x_lo];
    
    for j=1:binsHigh
        y_lo = floor(hgt/binsHigh * (j-1));
        y_hi = floor(hgt/binsHigh * j);    
        ycoords = [ycoords y_lo];
        
        texton_patch = texton_ind.data( (texton_ind.x > x_lo) & (texton_ind.x <= x_hi) & ...
            (texton_ind.y > y_lo) & (texton_ind.y <= y_hi));
            
        % make histogram of features in bin
        pyramid_cell{1}(i,j,:) = hist(texton_patch, 1:dictionarySize)./length(texton_ind.data);
    end
end

xcoords = repmat(xcoords,1,binsHigh);

%plot the grid
plotGrid(image, xcoords, ycoords, pyramidLevels);

%% compute histograms at the coarser levels
num_bins = binsHigh/2;
for l = 2:pyramidLevels
    pyramid_cell{l} = zeros(num_bins, num_bins, dictionarySize);
    for i=1:num_bins
        for j=1:num_bins
            pyramid_cell{l}(i,j,:) = ...
                pyramid_cell{l-1}(2*i-1,2*j-1,:) + pyramid_cell{l-1}(2*i,2*j-1,:) + ...
                pyramid_cell{l-1}(2*i-1,2*j,:) + pyramid_cell{l-1}(2*i,2*j,:);
        end
    end
    sp = 2^(l-1);
    plotGrid(image, xcoords(1:sp:end), ycoords(1:sp:end),pyramidLevels-l+1);
    num_bins = num_bins/2;
end

plotHists(pyramid_cell, pyramidLevels);


function [] = plotHists(pyramid_cell, pyramidLevels)
for l = 1:pyramidLevels,
    figure; %title(sprintf('Histograms for level %d',l));
    hold on;
    pc = pyramid_cell{l};
    [w,h,ds] = size(pc);
    for i = 1:w,
        for j = 1:h,
            subplot(w,h,(i-1)*h+j);
            bar(reshape(pc(i,j,:)*1000,1,ds));
            set(gca,'xlim',[1 ds],'XTick',[1 ds],'YTick',[]);
        end
    end
end


function [] = plotGrid(image, xcoords, ycoords, pyramidLevel)
figure;
imshow(image);
hold on;
title(sprintf('Pyramid level %d',pyramidLevel));
[width, height, c] = size(image);
for i = 1:size(xcoords,2),
    plot([xcoords(i) xcoords(i)],[0 width],'r-','LineWidth',2);
end
for i = 1:size(ycoords,2),
    plot([0 height], [ycoords(i) ycoords(i)],'r-','LineWidth',2);
end
